pro shapelets_focus_betax0,obj,nmax,shapelets_focus,decomp,recomp,beta_guess=beta_guess,x0_guess=x0_guess,$
    iter_max=iter_max,tol=tol,append=append,th_min=th_min,$
    psf=psf,printit=printit

;$Id: shapelets_focus_betax0.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; April 2002 - Minor additions and PSF deconvolution by R. Massey
; March 2002 - Written by A. Refregier
;
; PURPOSE: find the optimal beta and centroid x0 for the decomposition of
; an image by minimising Chi^2. This is done using a 1-dimensional Amoeba
; search. The maximum shapelet order nmax is assumed to be known.
; INPUT: obj: object structure from shapelets_extract_pstamp.pro
;        nmax: maximum shapelet order for decomposition
; OPTIONAL INPUT: x0_guess: guess for centroid (default:obj.x0)
;                 beta_guess: initial guess for beta (default: estimated)
;                 iter_max: maximum number of iterations (default:30)
;                 tol: Chi^2 tolerance for Amoeba search (default=1e-3)
;                 append: append the history file to the current
;                         shapelets_focus structure
;                 th_min: minimum scale theta_min (default:.5 pixels) 
; OUTPUT: decomp: decomposition at optimal beta and x0
;         shapelets_focus: structure containing the history of the search
;-

common betax0_common,obj_1,nmax_1,x0_1,hist,th_min_1,flag

; declarations
if not keyword_set(iter_max) then iter_max=30
if not keyword_set(tol) then tol=1e-2
n_h=iter_max+5    ; history vector length
if not keyword_set(th_min) then th_min=.5
if not keyword_set(append) then flag=0 else flag=shapelets_focus.flag
th_min_1=th_min
scale_fact=4.3

; guess beta and x0 if not provided
shapelets_guess_nmaxbeta,obj,guess;,/printit
if not keyword_set(beta_guess) then begin
  beta_guess=guess.beta
  if keyword_set(printit) then $
    print,'guess: beta:',beta_guess
endif
if not keyword_set(x0_guess) then begin
  x0_guess=guess.x0
  if keyword_set(printit) then $
    print,'guess: x0:',x0_guess(0),x0_guess(1) 
endif 
obj_1=obj
nmax_1=nmax
x0_1=x0_guess

; set scale for amoeba search
beta_max=min([x0_guess(0),obj.im_ran(1)-obj.im_ran(0)-x0_guess(0),$ 
            x0_guess(1),obj.im_ran(3)-obj.im_ran(2)-x0_guess(1)])$  ; th_max
		  /sqrt(nmax+1)
beta_min=(th_min*sqrt(nmax+1))
direction=sign(-2.*beta_guess+beta_max+beta_min) ; But which is it closer to? 
;print,"beta_guess   =",beta_guess
;print,"direction=",direction
scale=direction*beta_guess/scale_fact            ; Go in opposite direction!
;print,"scale    =",scale

; construct history structure
si=size(shapelets_focus,/st)
if keyword_set(append) and si.type ne 0 then begin
  n_shapelets_focus=n_elements(shapelets_focus.beta_h)
  n_hh=n_shapelets_focus+n_h
  i_h=n_shapelets_focus-1
  hist={i:i_h,beta:fltarr(n_hh),nmax:intarr(n_hh),chi2:fltarr(n_hh),$
    x0:fltarr(2,n_hh)}
  hist.beta(0:n_shapelets_focus-1)=shapelets_focus.beta_h
  hist.nmax(0:n_shapelets_focus-1)=shapelets_focus.nmax_h
  hist.chi2(0:n_shapelets_focus-1)=shapelets_focus.chi2_h
  hist.x0(*,0:n_shapelets_focus-1)=shapelets_focus.x0_h
endif else begin
  hist={i:-1,beta:fltarr(n_h),nmax:intarr(n_h),chi2:fltarr(n_h),$
    x0:fltarr(2,n_h)}
endelse

; find beta and x0 by minimizing chi^2
chi2_min=-1.
beta_min=shapelets_amoeba(tol,function_name='shapelets_amoeba_chi2',printit=printit,$
  nmax=iter_max,p0=beta_guess,scale=scale,function_value=chi2_min,flag=flag,psf=psf)

; make one last decomposition at the best beta and x0
shapelets_decomp,obj.image,beta_min,nmax,decomp,recomp,$
  x0=x0_1,/ls,/over,noise=obj.noise,/integrate,psf=psf
hist.i=hist.i+1
hist.beta(hist.i)=beta_min
hist.nmax(hist.i)=nmax
hist.x0(*,hist.i)=x0_1
hist.chi2(hist.i)=decomp.chisq(1)
if keyword_set(printit) then $
  print,'shapelets_focus_betax0: nmax, beta, Chi2, x0:', $
  nmax,beta_min,decomp.chisq[1],x0_1[0],x0_1[1]

; store results and history in structure
shapelets_focus={beta:beta_min,nmax:nmax,x0:x0_1,chi2:decomp.chisq(1),tol_chi2:tol,$
      th_min:th_min,chi20:-1,beta_guess:beta_guess,nmax_guess:nmax,n_iter:hist.i+1,x0_guess:x0_guess,$
      beta_h:hist.beta(0:hist.i),nmax_h:hist.nmax(0:hist.i),$
      chi2_h:hist.chi2(0:hist.i),x0_h:hist.x0(*,0:hist.i),$
	 flag:flag}

return
end
