pro shapelets_theta_minmax, beta, n_max, theta, reverse=reverse

;$Id: shapelets_theta_minmax.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; Dec 01  - Written by R. Massey
;
; PURPOSE:   Find theta_min and theta_max, given n_max and beta
;            (as prescribed in paper I)
;            If /reverse flag is set, find beta and n_max for shapelet 
;            decomposition given theta_min and theta_max.
; INPUT:     beta:  shapelet scale size
;            n_max: shapelet truncation parameter
; OUTPUT:    theta[min,max]
;-

if not keyword_set(reverse) then reverse=0

if reverse eq 0 then begin
  theta    = fltarr(2)
  theta(0) = float(beta)/sqrt(float(n_max)+.5)
  theta(1) = float(beta)*sqrt(float(n_max)+.5)
endif else begin
  beta     = sqrt( theta(0) * theta(1) )
  n_max    = round( theta(1) / theta(0) - .5 )
endelse

end
