function shapelets_convolve,decomp,psf,gamma=gamma,n_max_g=n_max_g

;$Id: shapelets_convolve.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; Written by Richard Massey - Mar 02
;
; Convolves an object, in Cartesian decomp structure form, with a general PSF,
; also as a Cartesian decomp structure.
;
; INPUTS: decomp: original object, as a Cartesian decomp structure
;            psf: PSF model, as a Cartesian decomp structure
;        [gamma]: gamma to use for convolved object
;                 default: "natural choice" of paperII fame
;        [n_max]: n_max to use for convolved object
;                 default: same as input object
;
; OUTPUT: returns convolved object, as a Cartesian decomp structure
;-

alpha=decomp.beta & beta=psf.beta
if not keyword_set(gamma) then gamma=sqrt(alpha^2+beta^2)
n_max_a=decomp.n_max
if not keyword_set(n_max_g) then n_max_g=decomp.n_max

P_nm=shapelets_convolution_matrix(psf,alpha,n_max_a,gamma,n_max_g)

shapelets_make_nvec, n_max_g, n1_g, n2_g, nn_g

f_m=decomp.coeffs
; Convolution (paperI eqn54)
h_n=P_nm#f_m

; long-winded check by hand
;h_n=fltarr(nn_g)
;h_error=fltarr(nn_g)
;for n=0,nn_g-1 do begin
;  for m=0,decomp.n_coeffs-1 do begin
;    h_n(n)=h_n(n)+P_nm(n,m)*f_m(m)
;    h_error(n)=h_error(n)+P_nm(n,m)*decomp.error(m)
;  endfor
;endfor

; Might as well do errors at the same time (do these transform in exactly the same way?)
;n_a=n_elements(decomp.coeffs)
;if n_elements(decomp.error) eq n_a $
;  then h_error=P_nm#decomp.error $
;  else h_error=fltarr(n_a)

output = {name:decomp.name,             $
		type:decomp.type, 			$
		history:decomp.history,		$
		x:decomp.x,                   $
		beta:gamma,				$
		n_max:n_max_g,				$
		n_coeffs:nn_g,  			$
		coeffs:h_n,     			$
		n_pixels:decomp.n_pixels,     $
		moments:decomp.moments,		$
		sex:decomp.sex,			$
;		w:gamma,					$ ; Obsolescent |
;		nmax:n_max_g,				$ ;  		 |
;		x0:decomp.x,                  $ ;  		 V
		n1:n1_g,					$
		n2:n2_g,					$
;		a:h_n, 					$
;		n_a:nn_g,  				$
;		nf:decomp.n_pixels,           $
		over:decomp.over,			$
		integrate:decomp.integrate,	$
		error:decomp.error,  		$
		chisq:decomp.chisq, 		$
		skyfit:decomp.skyfit}


return,output

end
