pro shapelets_shear,decomp,gammas_in,extend=extend

;$Id: shapelets_shear.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; Richard Massey - April 2002
;
; Shears an object to first order, using ladder operators.
; TO DO: implement in polar shapelet coefficient space with a switch.
;
; INPUT: decomp: A shapelet decomposition (unaffected except for a & e).
;        gammas: Input shear [gamma_1,gamma_2].
;      [extend]: Increses n_max by 2 (and sets new coefs to zero) before
;                starting to shear. Gives shear somewhere to go.
;
; OUTPUT: decomp is returned, still as a Cartesian decomp structure and with
;         the same beta as before, but sheared
;-

gammas=-gammas_in ; Convert to `industry standard`:
                  ; gamma1>0 => horizontal stretch
			   ; gamma2>0 => bl-tr stretch

if keyword_set(extend) then shapelets_extend_nmax,decomp,2

new_coeffs=decomp.coeffs
shapelets_make_nvec,decomp.n_max,n1,n2,n_coeffs

for i=0,decomp.n_coeffs-1 do begin

  ; S_1
  ; Expand leftwards in Cartesian shapelet coefficient space
  j=where(n1 eq n1(i)+2 and n2 eq n2(i))
  if j(0) ne -1 then new_coeffs(i)=new_coeffs(i)+gammas(0)/2.*sqrt((n1(i)+1)*(n1(i)+2))*decomp.coeffs(j(0))

  ; Expand downwards in Cartesian shapelet coefficient space
  j=where(n1 eq n1(i) and n2 eq n2(i)+2)
  if j(0) ne -1 then new_coeffs(i)=new_coeffs(i)-gammas(0)/2.*sqrt((n2(i)+1)*(n2(i)+2))*decomp.coeffs(j(0))

  ; Expand rightwards in Cartesian shapelet coefficient space
  j=where(n1 eq n1(i)-2 and n2 eq n2(i))
  if j(0) ne -1 then new_coeffs(i)=new_coeffs(i)-gammas(0)/2.*sqrt(n1(i)*(n1(i)-1))*decomp.coeffs(j(0))

  ; Expand upwards in Cartesian shapelet coefficient space
  j=where(n1 eq n1(i) and n2 eq n2(i)-2)
  if j(0) ne -1 then new_coeffs(i)=new_coeffs(i)+gammas(0)/2.*sqrt(n2(i)*(n2(i)-1))*decomp.coeffs(j(0))
 
  ; S_2
  ; Expand diagonally down-left in Cartesian shapelet coefficient space
  j=where(n1 eq n1(i)+1 and n2 eq n2(i)+1)
  if j(0) ne -1 then new_coeffs(i)=new_coeffs(i)+gammas(1)*sqrt((n1(i)+1)*(n2(i)+1))*decomp.coeffs(j(0))

  ; Expand diagonally up-right in Cartesian shapelet coefficient space
  j=where(n1 eq n1(i)-1 and n2 eq n2(i)-1)
  if j(0) ne -1 then new_coeffs(i)=new_coeffs(i)-gammas(1)*sqrt(n1(i)*n2(i))*decomp.coeffs(j(0))


endfor  

decomp.coeffs=new_coeffs

return
end

