pro plt_colbar,range,title=title,scalable=scalable,inverse=inverse,$
  csize=csize

;$Id: plt_colbar.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; October 1997 - Written by A. Refregier
;
; PURPOSE: draw an annotated color bar at the top of the plotting
; window. The window parameters are then set to the remainder of
; the window.
; INPUT: range: range of the scale for the color bar
; OPTIONAL INPUT: title: title for the color bar
; 		  scalable: use scalable pixels (useful when
;                           producing postcript files)
;                 inverse: invert the color coding
;                 csize: vertical color bar size (0-1)
; OUTPUT: an annotated color bar is drawn at the top of the window.
; The window parameters are then set to the remainder of the window.
; NOTE: to restore normal window region set !p.region=[0,0,0,0]
;-

;print,range

; set color bar vertical size
if not keyword_set(csize) then csize=.14

; reserve upper portion of the window for the color bar
!p.region=[0.,1.-csize,1.,1.]

; create a ramp for the color bar
colbar=fltarr(256,2)
colbar(*,0)=findgen(256)
colbar(*,1)=findgen(256)
if keyword_set(inverse) then colbar=-colbar
	
; plot the color bar
if not keyword_set(title) then title=''
plot,[0],[0],/nodata,xstyle=12,ystyle=12
plt_image,colbar,scalable=scalable,frame=-1
plot,[0],[0],/nodata,/noerase,xrange=range,yrange=[0.,1.],$
  ystyle=12,/xstyle,title=title
oplot,[.99999,.99999]*range(0),[0.,1.]
oplot,[.99999,.99999]*range(1),[0.,1.]

; leave the rest of the window for the image
!p.region=[0.,0.,1.,1.-csize]

end
