pro plt_pstamp,obj,oframe=oframe,noise=noise,title=title,cran=cran

;$Id: plt_pstamp.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; March 2002 - Written by A. Refregier
;
; PURPOSE: plot the image and sextractor parameters for an object
; INPUT: obj: object structure from shapelets_image2obj.pro
; OPTIONAL INPUT: oframe: label the plot with the object frame coordinates
;                 noise: plot noise or S/N for noise=1,2 respectively
; OUTPUT: plot
;-

; select function to plot
if not keyword_set(title) then title=''
if not keyword_set(noise) then begin
  f=obj.image
endif else begin
  if noise eq 1 then begin
    f=obj.noise
    title=title+' rms noise'
  endif else begin
    f=obj.image/obj.noise
    title=title+' S/N'
  endelse
endelse

; plot image and Sextractor parameters
if not keyword_set(oframe) then begin       ; image frame
  plt_image,f,/col,cran=cran,$
    frame=[obj.im_ran(0),obj.im_ran(1)+1,obj.im_ran(2),obj.im_ran(3)+1],$
    xtitle='x (pixels)',ytitle='y (pixels)',title=title
  ;plt_ellipse,obj.x,obj.y,$
  ;  obj.a,obj.b,(obj.theta+90.)/!radeg
  oplot,[obj.x],[obj.y],psym=1
endif else begin                            ; object frame
  plt_image,f,/frame,/col,cran=cran,$
    xtitle='x (pixels)',ytitle='y (pixels)',title=title  
  ;plt_ellipse,obj.xo,obj.yo,$
  ;  obj.a,obj.b,(obj.theta+90.)/!radeg
  oplot,[obj.xo],[obj.yo],psym=1
endelse

; SExtractor ellipse
plt_ellipse,obj.xo,obj.yo,2.*obj.a,2.*obj.b,(obj.theta+90)*!pi/180,linestyle=0
; SExtractor min/max pixels
plt_rectangle,[obj.xmin+obj.xo-obj.x,obj.ymin+obj.yo-obj.y], $
		[obj.xmax+obj.xo-obj.x,obj.ymax+obj.yo-obj.y],linestyle=2 

end
