function shapelets_paths, n, SILENT=silent

;$Id: shapelets_paths.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;       SHAPELTS_PATHS
;
; PURPOSE:
;       This stores the locations of data on a locally accesible hard disc.
;       Please update this file so that all the strings point to the correct
;       locations!
;
; CATEGORY:
;       Disc management.
;
; CALLING PROCEDURE:
;       filepath=shapelets_paths(1,/SILENT)
;
; INPUTS:
;       n - integer scalar. Which path?
;
; KEYWORD PARAMETERS:
;       [/SILENT] - If this flag is set, the routine operates silently.
;
; OUTPUTS:
;       A string containing a filepath.
;
; EXAMPLE:
;       Find the location where data is stored on the local disc.
;
;       filepath=shapelets_paths(1) & print,filepath
;
;       In this case (1), it is the base directory where data such as images,
;       shapelet/SExtractor catalogues or PSF images are stored.
;
; NOTES:
;       I have created this function so that the shapelet routines can be
;       quickly set up to run on a new computer, and only this file needs to
;       be altered.
;
; PROCEDURES USED:
;       Often calls itself recursively.
;
; MODIFICATION HISTORY:
;       Jul 03 - Finalised for version 1 of shapelet code release by RM.
;       Jan 01 - Written by Richard Massey
;-

message,'Shapelet export version of path names!',/info,noprint=silent

case n of

    9: begin  ; Base IDL directory
         case !version.os_family of
         'Windows': out='C:\Documents and Settings\Rich Massey\My Documents\IDL routines\shapelets\'
            'unix': out=expand_path('~')+'/idl/shapelets/'
              else: out=expand_path('~/idl/shapelets')+path_sep()
         endcase
       end

    1: $ ; Generic base data directory
       out=shapelets_paths(9,/silent)+'data'+path_sep()
       ;out="/disks/hebog3/rjm/shapelets/";shapelets_paths(9,/silent)+'data'+path_sep()

    2: $ ; Directory containing object catalogues (*.sex and *.shape)
       out=shapelets_paths(1,/silent)

    3: $ ; Directory containing images (*.fits)
       out=shapelets_paths(1,/silent)

    4: $ ; HST TinyTim PSF
       out=shapelets_paths(1,/silent)+expand_path('PSF/TinyTim_PSF.fits')

    5: $ ; Raytraced SNAP PSF
       out=shapelets_paths(1,/silent)+expand_path('PSF/SNAPpsf.fits')

    7: $ ; SExtractor directory
       out=expand_path('~/idl/shapelets/public_code/shapelets/shextractor/config')+path_sep()

    8: $ ; SExtractor executable
       out='/usr/local/optical/sextractor/bin/sex'
	  
 else: message,'Please set up shapelets_paths.pro to include the location of this file!'

endcase

return,out

end
