;$Id: shex.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
; ***********************************************************************
; ***********************************************************************
;
; NAME:
;       SHEX_PLOT
;
; PURPOSE:
;       Produces a few pretty plots to screen but does nothing else.
;
; CATEGORY:
;       A component of shex.pro
;
; INPUTS:
;       Various parameters to plot.
;
; OUTPUTS:
;       Plots to screen, asks user for input, more plots to screen!
;
; MODIFICATION HISTORY:
;       Sep 03 - Combined into shex.pro for shapelets code release
;       Dec 01 - Written by Richard Massey

pro shex_plot,image,decomp,x_bound,y_bound,x0_guess,i,a,b,theta,$
              xmin,xmax,ymin,ymax

;
; Set defaults
bit      = image(x_bound(0):x_bound(1), y_bound(0):y_bound(1))
beta     = decomp.beta
n_max    = decomp.n_max
x0       = decomp.x
x0_guess = x0_guess - [x_bound(0),y_bound(0)]
xsize    = x_bound(1)-x_bound(0)+1
ysize    = y_bound(1)-y_bound(0)+1
shapelets_theta_minmax,beta,n_max,thetaminmax
plotting:

; Display original image, with crosshairs
window,1,xsize=500,ysize=555,xpos=120,ypos=350
plt_image,bit,/col,title='!6Object #'+strtrim(i+1,1),/frame, $
  xtitle='pixels',ytitle='pixels';,/isotropic

; Display SExtractor guess
plot,[-900,900],[x0_guess(1),x0_guess(1)],/NOERASE, $
  xran=[0,xsize],yran=[0,ysize],/xstyle,/ystyle,linestyle=2
oplot,[x0_guess(0),x0_guess(0)],[-900,900],linestyle=2
plt_ellipse,x0_guess(0),x0_guess(1),2.*a,2.*b,theta,linestyle=2
plt_rectangle,[xmin-x_bound(0),ymin-y_bound(0)], $
              [xmax-x_bound(0),ymax-y_bound(0)],linestyle=2

; Display cenroid improved by shapelet iteration
oplot,[-900,900],[x0(1),x0(1)],thick=3
oplot,[x0(0),x0(0)],[-900,900],thick=3
circle,x0(0),x0(1),beta,thick=3
circle,x0(0),x0(1),thetaminmax(0),thick=3
circle,x0(0),x0(1),thetaminmax(1),thick=3

; Display coefficient matrix
window,0,xsize=500,ysize=555,xpos=630,ypos=350
plt_decomp,decomp,/coef,title='Shapelet coefficients and shapelet spectrum',$
  nran=[0,n_max],/pow

; Pause for user input, and give several options
read,"Please type a number to continue...",t

if ( t eq 2 ) then begin	         ; "2" to show more!

  ; Display reconstructed image with larger boundaries
  window,0,xsize=500,ysize=555,xpos=630,ypos=350
  coords=decomp.x
  decomp.x=decomp.n_pixels
  decomp.n_pixels=2*decomp.n_pixels
  shapelets_recomp,decomp,recomp
  ;recomp(where(recomp lt 0))=0.
  cran=[min([min(recomp),min(bit)]),max([max(recomp),max(bit)])]
  plt_image,recomp,title='Reconstructed image (& environs)',$
    /frame,cran=cran,/col,xtitle='pixels',ytitle='pixels'
  plt_rectangle,[xsize-coords(0),ysize-coords(1)],[2*xsize-coords(0),2*ysize-coords(1)],thick=3
  oplot,[xsize,xsize],[-900,ysize-coords(1)],thick=3
  oplot,[xsize,xsize],[2*ysize-coords(1),900],thick=3
  oplot,[-900,xsize-coords(0)],[ysize,ysize],thick=3
  oplot,[2*xsize-coords(0),900],[ysize,ysize],thick=3
  decomp.x=coords
  decomp.n_pixels=decomp.n_pixels/2.

  ; Display original image with colour rescaled to same range
  window,1,xsize=500,ysize=555,xpos=120,ypos=350
  plt_image,bit,title='Original image (rescaled)',$
    /frame,cran=cran,/col,xtitle='pixels',ytitle='pixels'

  ; Pause again for user input
  read,t

endif

; Parse other options
if ( t eq 0 ) then stop
if ( t eq 2 ) then goto,plotting

end







; ***********************************************************************
; ***********************************************************************
;
; NAME:
;       SHEX_RENUMBER
;
; PURPOSE:
;       This little subroutine is run at the end of shex.pro tp correct
;       the information in the header of a shapelet catalogue. If not all of
;       the objects found by SExtractor successfuly iterated into a shapelet
;       decomposition, they will be missing from the shapelet catalogue and the
;       number of objects claimed in the header (at the start of the file) will
;       be wrong. Since the file is written sequentially, this couldn't be
;       known when the numbers were written the first time.
;
;       This routine takes the correct numbers as an input (or works out
;       suitable numbers if they are not specified) and re-writes the entire
;       catalogue, including its header. This method is a little bit fiddly,
;       and is probably not the best solution to the problem, but it's the 
;       only one I could think of.
;
; CATEGORY:
;       A component of shex.pro
;
; INPUTS:
;       filename - Name of shapelets catalogue, complete with path and extension
;       n_tried  - Number of objects in SExtractor catalogue not near edge etc
;       n_objects- Number of successful decompositions (lines in shapelet cat)
;
; OUPUTS:
;       Alters catalogue file on disc.
;
; MODIFICATION HISTORY:
;       Sep 03 - Reconstructed entirely in IDL, to work on non-UNIX machines, by RM
;       Dec 01 - Original (using spawned C-shell commands) written by Richard Massey
;

pro shex_renumber,filename,n_tried,n_objects

COMPILE_OPT idl2

; Read in header information
resolve_routine, 'shapelets_read_shapecat', /COMPILE_FULL_FILE, /EITHER
;resolve_routine, 'shapelets_read_shapehead', /COMPILE_FULL_FILE, /EITHER
header_info=shapelets_read_shapehead(filename)

; Determine number of objects in catalogue if it wasn't entered as an argument
switch n_params() of
  1: n_tried=header_info.n_total
  2: n_objects=fix((file_lines(filename)-7)/3.)
endswitch
n_tried=n_tried>n_objects

; Read in original header as plain text strings
openr,lunr,filename,/get_lun
line1=strarr(1) & readf,lunr,line1
line2=strarr(1) & readf,lunr,line2
line3=strarr(1) & readf,lunr,line3
line4=strarr(1) & readf,lunr,line4
line5=strarr(1) & readf,lunr,line5
line6=strarr(1) & readf,lunr,line6
line7=strarr(1) & readf,lunr,line7

; Open temporary file to receive updated information
file_delete,filename+'.temp',/quiet
openw,lunw,filename+'.temp',/get_lun

; Write header to new file, with updated numbers
printf,lunw,line1
printf,lunw,line2
printf,lunw,format='("#",I11,3(I13),F13.6,2(I13),F13.6)',$
  n_objects,n_tried,header_info.n_max,header_info.n_coeffs,header_info.seeing,$
  header_info.n_pix[0],header_info.n_pix[1],header_info.pixel_scale
printf,lunw,line4
printf,lunw,line5
printf,lunw,line6
printf,lunw,line7

; Copy across the rest of the file: n*3 data lines + 1 end line
;line=strarr(1)
;for i=0L,n_objects*3 do begin
;  readf,lunr,line
;  printf,lunw,line
;endfor

; Better version just carries on until the end flag is reached
line="     "
while strmid(line,0,5) ne "# End" do begin
  readf,lunr,line
  printf,lunw,line
endwhile

; Close up both new and original files
close,lunw
free_lun,lunw
close,lunr
free_lun,lunr

; Replace the old catalogue with the new one
file_move,filename+'.temp',filename,/overwrite

end




; ***********************************************************************
; ***********************************************************************
;
;+
; NAME:
;       SHEX
;
; PURPOSE:
;       Having located objects within an image using SExtractor, this now
;       decomposes all of the objects into shapelets. It uses the shapelets
;       focus suite of routines to optimise the nmax, beta and centroid
;       parameters. All of the decompositions are written to disc in a
;       shapelets catalgoue. This can be later read to memory using
;       shapelets_read_catalgoue.pro
;
; CATEGORY:
;       ROI modelling.
;
; CALLING PROCEDURE:
;       shex,'example',sexcat=sexcat,n_max=15,/plot
;
; INPUTS:
;       filename - Name of input image.
;
; OPTIONAL INPUTS:
;       psf      - Shapelet decompstructure containing PSF to deconvolve from.
;                  Currently, this has to be the same everywhere on an image.
;	  image    - Can set in advance so it doesn't have to be loaded in.
;	  sexcat   - Can set in advance so it doesn't have to be loaded in.
;	  n_max    - Highest order of n_max=n1+n2 used for decomposition.
;	  n_min    - Minimum valus of n_max used for decomposition.
;                  Together, these two options specify the allowed range.
;       sg_cut   - Threshold level for star/galaxy separation, using the 
;                  SExtractor measure of stariness from {0,1}. Only galaxies 
;                  are subsequently decomposed. Default value is 0.97.
;
; KEYWORD PARAMETERS:
;	  /PLOT    - If set, will display pretty pictures until 9 is pressed.
;	  /SILENT  - Operates silently.
;	  /VERBOSE - Operates noisily.
;
; OUTPUTS:
;       Shapelet catalogue written to disc, with the same file name and path
;       as the input image, but with extension .shape.
;
;       Also, optionally:
;       cat      - Shapelet catalogue data structure stored to memory.
;                  View contents by typing help,cat,/structure
;	  image    - Image data structure, if none was entered originally.
;	  sexcat   - SExtractor catalogue data structure, if none was entered.
;
; DATA REQUIRED:
;        Needs an image and a SExtractor catalogue to initially locate the objects
;        for decomposition.
;
; PROCEDURES USED:
;        This combines most of the decomposition and focus routines in the shapelets
;        code release.
;
; NOTES:
;        The first of the two flags for each objet in a shapelet catalogue describes
;        its segmentation into a small postage stamp. The second describes its
;        decomposition into shapelets. Their precise meanings are given below, but see
;        the IDL headers in shapelets_image2obj.pro and shapelets_focus.pro
;        for further information.
;
;        MEANING OF FIRST FLAG:
;        0: OK
;        1: did not normalise noise locally (is object filling postage stamp?)
;        2: nearby object (beware overlapping isophotes!)
;        3: both of above
;        4: near edge or a saturated star
;        5: HUGE object (SExtractor gaves whole image occasionally)
;        NB the flag can only take one of these values.
;           If two conditions are duplicated, the higher number takes priority.
;
;        MEANINGS OF SECOND FLAG (from shapelets_focus.pro):
;        0: OK
;        1: has bounced into theta_min or max wall at some point
;        2: nmax_max reached - shapelets incompletely represent object
;        3: chi2>1 but converged (in n_max search) by flatness limit - still incomplete
;        4: did not converge
;        5: centroid off edge
;        6: fatal error
;        NB the flag can only take one of these values.
;           If two conditions are duplicated, the higher number takes priority.
;
; MODIFICATION HISTORY:
;        Oct 03 - Bug fix on centroiding by RM
;        Sep 03 - Converted for use on non-UNIX platforms by RM
;        Mar 03 - Significantly tidied up by RM
;        Aug 01 - Written by Richard Massey
;-

pro shex, FILENAME, PSF=psf, IMAGE=image, CAT=cat, SEXCAT=sexcat, $
          N_MIN=n_min, N_MAX=n_max, SG_CUT=sg_cut, $
          PLOTIT=plotit, SILENT=silent, VERBOSE=verbose

;
;
; SET DEFAULTS
;
;
if n_elements(silent) then if silent then begin & plotit=0 & verbose=0 & endif
if ( not n_elements(n_min) ) then n_min=2
if ( not n_elements(n_max) ) then n_max=15 else n_max = n_max < 43
shapelets_make_nvec,n_max,n1,n2,n_coeffs
filepath=shapelets_paths(1,/silent)                  ; Automatically prepend data directory opn file names
renorm  = 1                                          ; Locally renormalise inverse variance pixel noise map
sky     = 1	                                      ; Sky subtraction: 0=none 1=constant 2=plane 3=image renormalised
if not keyword_set(sg_cut) then sg_cut=0.97          ; Star-galaxy separation threshold
if not keyword_set(pixel_scale) then pixel_scale=0.  ; Size of pixels in arcseconds



;
;
; READ IN SExtractor CATALOGUE AND IMAGES
;
;
if keyword_set(dosex) or (not keyword_set(sexcat)) then $
  shapelets_read_sexcat,sexcat,filename,telescope=telescope,dosex=dosex,silent=silent
if keyword_set(dosex) or (not keyword_set(image)) then $
  shapelets_read_image,image,filename,sky=sky,silent=silent
n_pixels=size(image.image,/dimensions)


; Reorder the objects so we can look at the nice ones first if they are
; being displayed to screen, or other options.
n       = long(n_elements(sexcat.id)) ; Number of objects
;reorder = reverse(sort(sexcat.flux)) ; Look at pretty objects first...
;reorder = sort(sexcat.flux)          ; ...or low S/N ones first
reorder = indgen(n)                   ; ...or in SExtractor order
;reorder = reverse(indgen(n))         ; ...or in reverse SExtractor order


; Plot numbers of objects with detection significance
class = sexcat.class(reorder)
fwhm  = sexcat.fwhm(reorder)
mag   = sexcat.mag(reorder)
if (keyword_set(plotit) and n_elements(x) ge 2) then $
  plothist,snt,$;xran=[0,20],bin=1,$
  xtitle='!6Detection S/N',ytitle='Number of objects (total '+strtrim(n,2)+')'


; Determine seeing, and alter the filename if we are deconvolving from a PSF
if n_elements(psf) gt 0 then begin
  seeing=sqrt(shapelets_rsquared(psf))
  filename=filename+'_deconv'
endif else begin
  if ( max(class) gt 0.5 ) then $
       seeing=median(fwhm(where(class gt 0.5))) $
  else seeing=median(fwhm)
endelse

;filename=filename+'_nmaxmin=4'
;
;
; OPEN OUTPUT FILE:
;
;
openw,lun,shapelets_paths(3)+filename+'.shape',/get_lun
message,'Writing shapelet coefficients to '+filename+'.shape',$
  /info,noprint=silent


;
;
; WRITE OUT CATALOGUE HEADER (need to change the 999 if n_max>43):
;
;
printf,lun,'# Shapelet coefficients of objects in '+filename+'.fits: '
printf,lun,'#   num objs       out of     n_coeffs     n_coeffs       seeing  n_pixels(x)  n_pixels(y)  pixel scale'
printf,lun,format='("#",I11,3(I13),F13.6,2(I13),F13.6)',$
           n,n,n_max,n_coeffs,seeing,n_pixels[0],n_pixels[1],image.pixel_scale
printf,lun,'#'
printf,lun,'#         x0           y0         beta     SEx fwhm      SEx mag        n_max'+$
          '         Flag       SEx ID  Chi squared    SEx class        SEx A        SEx B'+$
	  '    SEx theta       SEx e1       SEx e2     SEx flux        SEx x        SEx y'+$
	  '     SEx area Object:x_min        x_max        y_min        y_max' 
printf,lun,format='($,"# n1",I8," ",(999(I12, :, " ")))',n1 & printf,lun
printf,lun,format='($,"# n2",I8," ",(999(I12, :, " ")))',n2 & printf,lun



;
;
; LOOP OVER ALL OBJECTS & DO DECOMPOSITIONS:
;
;
done =0L
tried=0L
for i=0L,n-1 do begin

  ; Only do galaxies (SEx.class is a measure of star-iness)
  if sexcat.class(reorder(i)) gt sg_cut then continue

  ;
  ;
  ; EXTRACT SMALL POSTAGE STAMP FROM LARGER IMAGE:
  ;
  ;
  shapelets_sexcat2pstamp,image,sexcat,reorder(i),obj,plotit=plotit,$
                      renorm=(strmid(filename,0,3) eq "HDF");,/silent
  if obj.flag ge 4 then message,$
    'Skipping object with flag='+(strtrim(obj.flag,2)),/info,noprint=silent
  if obj.flag ge 4 then continue ; Skip objects near edge of image, etc.


  ; Postage stamp looks OK: carry on
  tried=tried+1
  message,'Decomposing object #'+strtrim(reorder(i),1)+', '+strtrim(i+1,1)+'/'+strtrim(n,1)+ $
    ' at ('+strtrim(round(sexcat.x(reorder(i),0)),1)+','+strtrim(round(sexcat.x(reorder(i),1)),1)+').' $
    ;+' S/N = '+strtrim(sexcat.flux(reorder(i)),1) $
    ,/info,noprint=silent


  ;
  ;
  ; OPTIMISE ("FOCUS") SHAPELET DECOMPOSITION PARAMETERS n_max, beta AND x_c:
  ;
  ;
  shapelets_focus,obj,shapelets_focus,decomp,recomp,printit=verbose,silent=silent,$
  nmax_ran=[n_min,n_max],psf=psf,th_min_geom=0.2,nmax_g=2>n_min
  x0=shapelets_focus.x0+[obj.x,obj.y]-[obj.xo,obj.yo]
  if shapelets_focus.flag ge 4 then continue   ; Skip objects that did not converge
  if shapelets_flux(decomp) le 0 then continue ; Skip objects with negative flux


  ;
  ;
  ; WRITE RESULTS TO CATALOGUE
  ;
  ;
  coeffs   = decomp.coeffs
  errors   = decomp.error
  otherdata= [x0,                     $
             decomp.beta,             $
	        sexcat.fwhm[reorder[i]], $
	        sexcat.mag[reorder[i]],  $
	        decomp.n_max,            $
	        10*obj.flag+shapelets_focus.flag, $
	        sexcat.id[reorder[i]],   $
	        decomp.chisq[1],         $
             sexcat.class[reorder[i]],$
             sexcat.a[reorder[i]],    $
             sexcat.b[reorder[i]],    $
             sexcat.theta[reorder[i]],$
             sexcat.e1[reorder[i]],   $
             sexcat.e2[reorder[i]],   $
             sexcat.flux[reorder[i]], $
             sexcat.x[reorder[i],0],  $
             sexcat.x[reorder[i],1],  $
             sexcat.area[reorder[i]], $
             obj.im_ran[0],           $
             obj.im_ran[1],           $
             obj.im_ran[2],           $
             obj.im_ran[3] ]
  if n_coeffs gt 0 then coeffs[1:*]=coeffs[1:*]/coeffs[0]
  while ( n_elements(coeffs) lt n_coeffs ) do begin
    coeffs=[coeffs,0.]
    errors=[errors,0.]
  endwhile
  printf,lun,format='($,(5(F12.5, :, " ")),(3(I12, :, " ")),(10(F12.4, :, " ")),(5(I12, :, " ")))',otherdata & printf,lun
  printf,lun,format='($,(999(E12.5, :, " ")))',coeffs  & printf,lun
  printf,lun,format='($,(999(E12.5, :, " ")))',errors  & printf,lun
  done=done+1


  ; Plot to screen if required
  if keyword_set(plotit) then begin
    plt_focus,obj,shapelets_focus,decomp
    read,'Type a number to continue...',junk
    case junk of
        0: break
        2: writefits,'postagestamp'+strtrim(reorder[i],2)+'.fits',obj.im
        9: plotit=0
     else:
    endcase
  endif

endfor


; Write end flag to bottom of shapelet catalogue
printf,lun,'# End'
close,lun
free_lun,lun


; Number of objects in the catalogue's header is wrong if not all of them were
; successfully decomposed into shapelets. This routine updates that number.
if done ne n then shex_renumber,shapelets_paths(3)+filename+'.shape',tried,done


; Read whole shapelet catalogue in to memory
if keyword_set(cat) then shapelets_read_shapecat,cat,filename,/moments


; Write final message to screen proclaiming success!
message,'Finished '+filename+'.shape',/info,noprint=silent

end
